(************************************************************* 
			Mini Java 
 *************************************************************
 Auteur :	M. Sighireanu
 Fichier :	javalexer.mll
 Version :	1.2
 Date :		00/01/21 10:08:01
 *************************************************************
 Description :	lexer Caml pour  Mini Java
 *************************************************************)

{
open Javaparser
open List
exception Eof
exception InternalError of string


(*
 ** Keyword table
 *)
let keywords =
	[
		("class", CLASS);
		("extends", EXTENDS);
		("boolean", BOOL); 		(* Types and values*)
		("int", INT);
		("void", VOID);
		("public", PUBLIC);		(* Modifiers *)
		("protected", PROTECTED);
		("private", PRIVATE);
		("static", STATIC);
		("if", IF);			(* Statements *)
		("else", ELSE);
		("for", FOR);
		("while", WHILE);
		("return", RETURN);
		("new", NEW);			(* Expressions *)
		("null", NULL);
		("this", THIS);
		("false", FALSE);
		("true", TRUE);
	]
;;

let scan_ident ident =
  try assoc ident keywords
  with _ -> IDENT ident;;
}

let decdigit = ['0'-'9']
let octdigit = ['0'-'7']
let hexdigit = ['0'-'9' 'a'-'f' 'A'-'F']
let letter = ['a'- 'z' 'A'-'Z']


let intnum = decdigit+

let exponent = ['e' 'E']['+' '-']? decdigit+
let fraction  = '.' decdigit+
let floatnum = (intnum? fraction)
			|(intnum exponent)
			|(intnum? fraction exponent)
			|(intnum '.') 

let ident = (letter|'_')(letter|decdigit|'_')* 
let comment = "(*" _* "*)"
let blank = [' ' '\t' ] | comment

rule initial = parse
			blank		{initial lexbuf} (* skip blanks*)
	|		'\n'		{Javasyntax.noline := !Javasyntax.noline + 1; initial lexbuf}
	|		intnum		{CST_INT (Lexing.lexeme lexbuf)}
	|		"=="		{EQ_EQ}
	|		"!="		{EXCLAM_EQ}
	|		"<="		{INF_EQ}
	|		">="		{SUP_EQ}
	|		"="		{EQ}
	|		"<"		{INF}
	|		">"		{SUP}
	|		'+'		{PLUS}
	|		'-'		{MINUS}
	|		'*'		{STAR}
	|		'/'		{SLASH}
	|		'%'		{PERCENT}
	|		'!'		{EXCLAM}
	|		"&&"		{AND_AND}
	|		"||"		{PIPE_PIPE}
	|		':'		{COLON}
	|		'{'		{LBRACE}
	|		'}'		{RBRACE}
	|		'['		{LBRACKET}
	|		']'		{RBRACKET}
	|		'('		{LPAREN}
	|		')'		{RPAREN}
	|		';'		{SEMICOLON}
	|		','		{COMMA}
	|		'.'		{DOT}
	|		ident		{scan_ident (Lexing.lexeme lexbuf)}
	|		eof		{Javasyntax.noline := 0; EOF}
	|		_		{raise (Failure "lex")}

	


